"""Schemes"""

import os
import xml.etree.ElementTree as ET

# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-branches


class Action:
    """Base class for actions."""

    def __init__(self, node) -> None:
        self.command_line_arguments = []
        self.environment_variables = []
        self.additional_options = []
        self.pre_actions = []
        self.post_actions = []
        self._understood_tags = {
            "CommandLineArguments",
            "EnvironmentVariables",
            "MacroExpansion",
            "AdditionalOptions",
            "PreActions",
            "PostActions",
            "buildConfiguration",
        }

        self._understood_attributes = {
            "enableAddressSanitizer",
            "enableThreadSanitizer",
            "enableUBSanitizer",
            "disableMainThreadChecker",
            "enableASanStackUseAfterReturn",
            "enableGPUValidationMode",
            "migratedStopOnEveryIssue",
            "language",
            "region",
            "allowLocationSimulation",
            "askForAppToLaunch",
            "launchAutomaticallySubstyle",
        }
        self.build_configuration = node.attrib.pop("buildConfiguration", None)
        self.enable_address_sanitizer = node.attrib.pop("enableAddressSanitizer", None) == "YES"
        self.enable_thread_sanitizer = node.attrib.pop("enableThreadSanitizer", None) == "YES"
        self.enable_ub_sanitizer = node.attrib.pop("enableUBSanitizer", None) == "YES"
        self.enable_asan_stack_use_after_return = (
            node.attrib.pop("enableASanStackUseAfterReturn", None) == "YES"
        )
        self.enable_gpu_validation_mode = node.attrib.pop("enableGPUValidationMode", None)
        self.disable_main_thread_checker = (
            node.attrib.pop("disableMainThreadChecker", None) == "YES"
        )
        self.migrated_stop_on_every_issue = (
            node.attrib.pop("migratedStopOnEveryIssue", None) == "YES"
        )
        self.language = node.attrib.pop("language", None)
        self.region = node.attrib.pop("region", None)
        self.allow_location_simulation = node.attrib.pop("allowLocationSimulation", None) == "YES"
        self.ask_for_app_to_launch = node.attrib.pop("askForAppToLaunch", None) == "YES"
        self.launch_automatically_substyle = node.attrib.pop("launchAutomaticallySubstyle", None)

        for child in node:
            if child.tag == "CommandLineArguments":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for argument in child:
                    if argument.tag == "CommandLineArgument":
                        self.command_line_arguments.append(CommandLineArgument(argument))
                    else:
                        assert False, f"Unknown child: {argument.tag}"
            elif child.tag == "EnvironmentVariables":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for variable in child:
                    if variable.tag == "EnvironmentVariable":
                        self.environment_variables.append(EnvironmentVariable(variable))
                    else:
                        assert False, f"Unknown child: {variable.tag}"
            elif child.tag == "MacroExpansion":
                self.macro_expansion = MacroExpansion(child)
            elif child.tag == "AdditionalOptions":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for option in child:
                    if option.tag == "AdditionalOption":
                        self.additional_options.append(AdditionalOption(option))
                    else:
                        assert False, f"Unknown child: {variable.tag}"
            elif child.tag == "PreActions":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for action in child:
                    if action.tag == "ExecutionAction":
                        self.pre_actions.append(ExecutionAction(action))
                    else:
                        assert False, f"Unknown child: {variable.tag}"
            elif child.tag == "PostActions":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for action in child:
                    if action.tag == "ExecutionAction":
                        self.post_actions.append(ExecutionAction(action))
                    else:
                        assert False, f"Unknown child: {variable.tag}"

    def _understands_tag(self, tag: str) -> bool:
        return tag in self._understood_tags


class RunAction(Action):
    """Base class for runnable actions."""

    def __init__(self, node) -> None:
        super().__init__(node)
        self._understood_tags.add("RemoteRunnable")

        for child in node:
            if child.tag == "RemoteRunnable":
                self.remote_runnable = RemoteRunnable(child)


class BuildableReference:
    """BuildableReference"""

    def __init__(self, node) -> None:
        assert node.tag == "BuildableReference"
        self.buildable_identifier = node.attrib.pop("BuildableIdentifier", None)
        self.blueprint_identifier = node.attrib.pop("BlueprintIdentifier", None)
        self.buildable_name = node.attrib.pop("BuildableName", None)
        self.blueprint_name = node.attrib.pop("BlueprintName", None)
        self.referenced_container = node.attrib.pop("ReferencedContainer", None)

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert False, f"Unknown child: {child.tag}"


class BuildActionEntry:
    """BuildActionEntry"""

    def __init__(self, node) -> None:
        self.build_for_testing = node.attrib.pop("buildForTesting", None) == "YES"
        self.build_for_running = node.attrib.pop("buildForRunning", None) == "YES"
        self.build_for_profiling = node.attrib.pop("buildForProfiling", None) == "YES"
        self.build_for_archiving = node.attrib.pop("buildForArchiving", None) == "YES"
        self.build_for_analyzing = node.attrib.pop("buildForAnalyzing", None) == "YES"
        self.buildable_references = []

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            else:
                assert False, f"Unknown child: {child.tag}"


class MacroExpansion:
    """MacroExpansion"""

    def __init__(self, node) -> None:
        self.buildable_references = []

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            else:
                assert False, f"Unknown child: {child.tag}"


class CodeCoverageTargets:
    """CodeCoverageTargets"""

    def __init__(self, node) -> None:
        self.buildable_references = []

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            else:
                assert False, f"Unknown child: {child.tag}"


class LocationScenarioReference:
    """LocationScenarioReference"""

    def __init__(self, node) -> None:
        assert node.tag == "LocationScenarioReference"
        self.identifier = node.attrib.pop("identifier", None) == "YES"
        self.reference_type = node.attrib.pop("referenceType", None) == "YES"
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"


class Test:
    """Test"""

    def __init__(self, node) -> None:
        self.identifier = node.attrib.pop("Identifier", None)
        assert self.identifier is not None, "Missing identifier for Test"
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert False, f"Unknown child: {child.tag}"


class TestableReference:
    """TestableReference"""

    def __init__(self, node) -> None:
        self.skipped = node.attrib.pop("skipped", None) == "YES"
        self.parallelizable = node.attrib.pop("parallelizable", None) == "YES"
        self.use_test_selection_whitelist = (
            node.attrib.pop("useTestSelectionWhitelist", None) == "YES"
        )
        self.test_execution_ordering = node.attrib.pop("testExecutionOrdering", None)
        self.buildable_references = []
        self.selected_tests = []
        self.skipped_tests = []
        self.location_scenario_reference = None

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            elif child.tag == "SelectedTests":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for test in child:
                    if test.tag == "Test":
                        self.selected_tests.append(Test(test))
                    else:
                        assert False, f"Unknown child: {test.tag}"
            elif child.tag == "SkippedTests":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for test in child:
                    if test.tag == "Test":
                        self.skipped_tests.append(Test(test))
                    else:
                        assert False, f"Unknown child: {test.tag}"
            elif child.tag == "LocationScenarioReference":
                self.location_scenario_reference = LocationScenarioReference(child)
            else:
                assert False, f"Unknown child: {child.tag}"


class TestPlanReference:
    """TestPlanReference"""

    def __init__(self, node) -> None:
        assert node.tag == "TestPlanReference"
        self.reference = node.attrib.pop("reference", None)
        self.default = node.attrib.pop("default", None) == "YES"
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert False, f"Unknown child: {child.tag}"


class RemoteRunnable:
    """RemoteRunnable"""

    def __init__(self, node) -> None:
        self.runnable_debugging_mode = node.attrib.pop("runnableDebuggingMode", None)
        self.bundle_identifier = node.attrib.pop("BundleIdentifier", None)
        self.remote_path = node.attrib.pop("RemotePath", None)
        self.buildable_references = []
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            else:
                assert False, f"Unknown child: {child.tag}"


class KV:
    """KV"""

    def __init__(self, node) -> None:
        self.key = node.attrib.pop("key", None)
        self.value = node.attrib.pop("value", None)
        self.is_enabled = node.attrib.pop("isEnabled", None) == "YES"
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert False, f"Unknown child: {child.tag}"


class EnvironmentVariable(KV):
    """EnvironmentVariable"""

    def __init__(self, node) -> None:
        assert node.tag == "EnvironmentVariable"
        super().__init__(node)


class AdditionalOption(KV):
    """AdditionalOption"""

    def __init__(self, node) -> None:
        assert node.tag == "AdditionalOption"
        super().__init__(node)


class BuildableProductRunnable:
    """BuildableProductRunnable"""

    def __init__(self, node) -> None:
        assert node.tag == "BuildableProductRunnable"
        self.runnable_debugging_mode = node.attrib.pop("runnableDebuggingMode", None)
        self.buildable_references = []
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            else:
                assert False, f"Unknown child: {child.tag}"


class CommandLineArgument:
    """CommandLineArgument"""

    def __init__(self, node) -> None:
        assert node.tag == "CommandLineArgument"
        self.argument = node.attrib.pop("argument", None)
        self.is_enabled = node.attrib.pop("isEnabled", None) == "YES"
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert False, f"Unknown child: {child.tag}"


class EnvironmentBuildable:
    """EnvironmentBuildable"""

    def __init__(self, node) -> None:
        assert node.tag == "EnvironmentBuildable"
        self.buildable_references = []

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableReference":
                self.buildable_references.append(BuildableReference(child))
            else:
                assert False, f"Unknown child: {child.tag}"


class ActionContent:
    """ActionContent"""

    def __init__(self, node) -> None:
        assert node.tag == "ActionContent"
        self.title = node.attrib.pop("title", None)
        self.script_text = node.attrib.pop("scriptText", None)
        self.environment_buildable = None
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "EnvironmentBuildable":
                self.environment_buildable = EnvironmentBuildable(child)
            else:
                assert False, f"Unknown child: {child.tag}"


class ExecutionAction:
    """ExecutionAction"""

    def __init__(self, node) -> None:
        assert node.tag == "ExecutionAction"
        self.action_type = node.attrib.pop("ActionType", None)
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"
        self.action_content = None

        for child in node:
            assert self.action_content is None
            if child.tag == "ActionContent":
                self.action_content = ActionContent(child)
            else:
                assert False, f"Unknown child: {child.tag}"


class BuildAction(Action):
    """BuildAction"""

    def __init__(self, node) -> None:
        assert node.tag == "BuildAction"
        super().__init__(node)
        self.parallelize_buildables = node.attrib.pop("parallelizeBuildables", None) == "YES"
        self.build_implicit_dependencies = (
            node.attrib.pop("buildImplicitDependencies", None) == "YES"
        )
        self.build_action_entries = []
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildActionEntries":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for entry in child:
                    if entry.tag == "BuildActionEntry":
                        self.build_action_entries.append(BuildActionEntry(entry))
                    else:
                        assert False, f"Unknown child: {entry.tag}"
            else:
                assert self._understands_tag(child.tag)


class TestAction(Action):
    """TestAction"""

    def __init__(self, node) -> None:
        assert node.tag == "TestAction"
        super().__init__(node)
        self.build_configuration = node.attrib.pop("buildConfiguration", None)
        self.selected_debugger_identifier = node.attrib.pop("selectedDebuggerIdentifier", None)
        self.selected_launcher_identifier = node.attrib.pop("selectedLauncherIdentifier", None)
        self.should_use_launch_scheme_args_env = (
            node.attrib.pop("shouldUseLaunchSchemeArgsEnv", None) == "YES"
        )
        self.code_coverage_enabled = node.attrib.pop("codeCoverageEnabled", None) == "YES"
        self.system_attachment_lifetime = node.attrib.pop("systemAttachmentLifetime", None)
        self.only_generate_coverage_for_specific_targets = (
            node.attrib.pop("onlyGenerateCoverageForSpecifiedTargets", None) == "YES"
        )
        self.test_plans = []
        self.testables = []
        self.code_coverage_targets = None
        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "CodeCoverageTargets":
                self.code_coverage_targets = CodeCoverageTargets(child)
            elif child.tag == "Testables":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for testable in child:
                    if testable.tag == "TestableReference":
                        self.testables.append(TestableReference(testable))
                    else:
                        assert False, f"Unknown child: {testable.tag}"
            elif child.tag == "TestPlans":
                assert len(child.attrib) == 0, f"Unhandled attributes: {list(child.attrib.keys())}"
                for plan in child:
                    if plan.tag == "TestPlanReference":
                        self.test_plans.append(TestPlanReference(plan))
                    else:
                        assert False, f"Unknown child: {plan.tag}"
            else:
                assert self._understands_tag(child.tag)


class LaunchAction(RunAction):
    """LaunchAction"""

    def __init__(self, node) -> None:
        assert node.tag == "LaunchAction"
        super().__init__(node)
        self.build_configuration = node.attrib.pop("buildConfiguration", None)
        self.selected_debugger_identifier = node.attrib.pop("selectedDebuggerIdentifier", None)
        self.selected_launcher_identifier = node.attrib.pop("selectedLauncherIdentifier", None)
        self.launch_style = node.attrib.pop("launchStyle", None)
        self.use_custom_working_directory = (
            node.attrib.pop("useCustomWorkingDirectory", None) == "YES"
        )
        self.ignores_persistent_state_on_launch = (
            node.attrib.pop("ignoresPersistentStateOnLaunch", None) == "YES"
        )
        self.debug_document_versioning = node.attrib.pop("debugDocumentVersioning", None) == "YES"
        self.debug_service_extension = node.attrib.pop("debugServiceExtension", None)
        self.debug_xpc_services = node.attrib.pop("debugXPCServices", None) == "YES"
        self.language = node.attrib.pop("language", None)
        self.notification_payload_file = node.attrib.pop("notificationPayloadFile", None)
        self.console_mode = node.attrib.pop("consoleMode", None)
        self.buildable_product_runnable = None
        self.stop_on_every_main_thread_checker_issue = (
            node.attrib.pop("stopOnEveryMainThreadCheckerIssue", None) == "YES"
        )

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableProductRunnable":
                self.buildable_product_runnable = BuildableProductRunnable(child)
            else:
                assert self._understands_tag(child.tag)


class ProfileAction(RunAction):
    """ProfileAction"""

    def __init__(self, node) -> None:
        assert node.tag == "ProfileAction"
        super().__init__(node)
        self.build_configuration = node.attrib.pop("buildConfiguration", None)
        self.should_use_launch_scheme_args_env = (
            node.attrib.pop("shouldUseLaunchSchemeArgsEnv", None) == "YES"
        )
        self.saved_tool_identifier = node.attrib.pop("savedToolIdentifier", None)
        self.use_custom_working_directory = (
            node.attrib.pop("useCustomWorkingDirectory", None) == "YES"
        )
        self.debug_document_versioning = node.attrib.pop("debugDocumentVersioning", None) == "YES"
        self.launch_automatically_substyle = (
            node.attrib.pop("launchAutomaticallySubstyle", None) == "YES"
        )
        self.buildable_product_runnable = None

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildableProductRunnable":
                self.buildable_product_runnable = BuildableProductRunnable(child)
            else:
                assert self._understands_tag(child.tag)


class AnalyzeAction(Action):
    """AnalyzeAction"""

    def __init__(self, node) -> None:
        assert node.tag == "AnalyzeAction"
        super().__init__(node)
        self.build_configuration = node.attrib.pop("buildConfiguration", None)

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert self._understands_tag(child.tag)


class ArchiveAction(Action):
    """ArchiveAction"""

    def __init__(self, node) -> None:
        assert node.tag == "ArchiveAction"
        super().__init__(node)
        self.build_configuration = node.attrib.pop("buildConfiguration", None)
        self.reveal_archive_in_organizer = (
            node.attrib.pop("revealArchiveInOrganizer", None) == "YES"
        )
        self.custom_archive_name = node.attrib.pop("customArchiveName", None)

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            assert self._understands_tag(child.tag)


class Scheme:
    """Represents an Xcode scheme."""

    def __init__(self, node, name: str) -> None:
        assert node.tag == "Scheme"
        self.name = name
        self.last_upgrade_version = node.attrib.pop("LastUpgradeVersion", None)
        self.version = node.attrib.pop("version", None)
        self.was_created_for_app_extension = (
            node.attrib.pop("wasCreatedForAppExtension", None) == "YES"
        )
        self.build_action = None
        self.test_action = None
        self.launch_action = None
        self.profile_action = None
        self.analyze_action = None
        self.archive_action = None

        assert len(node.attrib) == 0, f"Unhandled attributes: {list(node.attrib.keys())}"

        for child in node:
            if child.tag == "BuildAction":
                self.build_action = BuildAction(child)
            elif child.tag == "TestAction":
                self.test_action = TestAction(child)
            elif child.tag == "LaunchAction":
                self.launch_action = LaunchAction(child)
            elif child.tag == "ProfileAction":
                self.profile_action = ProfileAction(child)
            elif child.tag == "AnalyzeAction":
                self.analyze_action = AnalyzeAction(child)
            elif child.tag == "ArchiveAction":
                self.archive_action = ArchiveAction(child)
            else:
                assert False, f"Unknown child: {child.tag}"

    @staticmethod
    def from_file(path: str) -> "Scheme":
        """Load a scheme from a file.

        :param path: The path of the scheme file

        :returns: A loaded scheme
        """
        tree = ET.parse(path)
        root = tree.getroot()

        return Scheme(root, ".".join(os.path.basename(path).split(".")[:-1]))

    @staticmethod
    def from_string(contents: str, name: str) -> "Scheme":
        """Load a scheme from a string.

        :param contents: The XML string
        :param name: The name of the scheme

        :returns: A loaded scheme
        """
        root = ET.fromstring(contents)
        return Scheme(root, name)



"""PBX build phases"""

from typing import List

import deserialize

from .pbxobject import PBXObject


@deserialize.auto_snake()
@deserialize.parser("isEditable", lambda x: {"0": False, "1": True}[x])
@deserialize.downcast_identifier(PBXObject, "PBXBuildRule")
class PBXBuildRule(PBXObject):
    """Represents a PBXBuildRule."""

    compiler_spec: str
    file_type: str
    input_files: List[str]
    output_files: List[str]
    is_editable: bool
    script: str



"""PBX path objects"""

import os
from typing import cast, List, Optional

import deserialize

from .pbxobject import PBXObject

# pylint: disable=no-member


@deserialize.key("source_tree", "sourceTree")
@deserialize.ignore("_parent_group_reference")
@deserialize.downcast_identifier(PBXObject, "PBXPathObject")
class PBXPathObject(PBXObject):
    """Represents an object with a path (i.e. file or group)."""

    path: Optional[str]
    source_tree: str

    _parent_group_reference: Optional[str]
    _full_path: Optional[str]

    def parent_group(self) -> Optional["PBXGroup"]:
        """Find the parent group of a reference.

        If a reference happens to be in multiple groups, only the first found
        instance will be returned.

        :returns: The parent group if found, None otherwise
        """

        if (
            hasattr(self, "_parent_group_reference")
            and getattr(self, "_parent_group_reference") is not None
        ):
            return cast(PBXGroup, self.project().objects[getattr(self, "_parent_group_reference")])

        for group in self.project().fetch_type(PBXGroup).values():
            if self in group.children:
                setattr(self, "_parent_group_reference", group.object_key)
                return group

        for group in self.project().fetch_type(PBXVariantGroup).values():
            if self in group.children:
                setattr(self, "_parent_group_reference", group.object_key)
                return group

        for group in self.project().fetch_type(XCVersionGroup).values():
            if self in group.children:
                setattr(self, "_parent_group_reference", group.object_key)
                return group

        return None

    def relative_path(self) -> Optional[str]:
        """Get the relative path for the group to the source root

        This works by getting the current references path, then searching up the
        tree, constructing the path as we go, until the root object is found.

        NOTE: This may contain a variable such as `$(BUILT_PRODUCTS_DIR)`

        :raises Exception: If an unexpected source tree is found

        :returns: The path if found, None otherwise
        """

        # pylint: disable=too-many-return-statements

        if hasattr(self, "_relative_path"):
            cached_path = getattr(self, "_relative_path")
            if cached_path is not None:
                return cast(str, cached_path)

        if self.source_tree == "SOURCE_ROOT":
            setattr(self, "_relative_path", self.path)
            return cast(str, self.path)

        if self.source_tree != "<group>":
            if self.source_tree in ["BUILT_PRODUCTS_DIR", "SDKROOT", "DEVELOPER_DIR"]:
                return f"$({self.source_tree})"
            raise Exception(f"Unexpected source tree: {self.source_tree}")

        parent = self.parent_group()

        if not parent:
            return None

        parent_path = parent.relative_path()

        if self.path is None:
            setattr(self, "_relative_path", parent_path)
            return parent_path

        if parent_path is None:
            setattr(self, "_relative_path", self.path)
            return self.path

        value = os.path.join(parent_path, self.path)
        setattr(self, "_relative_path", value)

        return value

        # pylint: enable=too-many-return-statements

    def absolute_path(self) -> Optional[str]:
        """Find the absolute path

        :returns: The absolute path of the object
        """

        path = self.relative_path()

        if path is None:
            return None

        return os.path.join(self.project().source_root, path)


@deserialize.key("indent_width", "indentWidth")
@deserialize.key("tab_width", "tabWidth")
@deserialize.key("uses_tabs", "usesTabs")
@deserialize.key("children_ids", "children")
@deserialize.ignore("_path")
@deserialize.parser("indentWidth", lambda x: int(x) if x else None)
@deserialize.parser("tabWidth", lambda x: int(x) if x else None)
@deserialize.parser("usesTabs", lambda x: int(x) > 0 if x else False)
@deserialize.downcast_identifier(PBXObject, "PBXGroup")
class PBXGroup(PBXPathObject):
    """Represents a PBXGroup.

    This is a group in the Xcode file explorer.
    """

    children_ids: List[str]
    indent_width: Optional[int]
    tab_width: Optional[int]
    uses_tabs: Optional[bool]
    name: Optional[str]

    @property
    def children(self) -> List[PBXPathObject]:
        """Get all the children for this group.

        :returns: The children for this group
        """
        return [cast(PBXPathObject, self.objects()[child_id]) for child_id in self.children_ids]


@deserialize.downcast_identifier(PBXObject, "PBXVariantGroup")
class PBXVariantGroup(PBXGroup):
    """Represents a PBXVariantGroup.

    These are similar to groups but are not backed by a folder on disk. This may
    be a deliberate choice, or it may be used to represent something like a
    .strings file, where the "file" can be expanded to see each language.
    """

    name: str


@deserialize.key("file_encoding", "fileEncoding")
@deserialize.key("last_known_file_type", "lastKnownFileType")
@deserialize.key("line_ending", "lineEnding")
@deserialize.key("indent_width", "indentWidth")
@deserialize.key("tab_width", "tabWidth")
@deserialize.key("xc_language_specification_identifier", "xcLanguageSpecificationIdentifier")
@deserialize.key("explicit_file_type", "explicitFileType")
@deserialize.key("include_in_index", "includeInIndex")
@deserialize.parser("fileEncoding", lambda x: int(x) if x else None)
@deserialize.downcast_identifier(PBXObject, "PBXFileReference")
class PBXFileReference(PBXPathObject):
    """Represents a PBXFileReference.

    The details of a particular file.
    """

    file_encoding: Optional[int]
    last_known_file_type: Optional[str]
    line_ending: Optional[str]
    indent_width: Optional[str]
    tab_width: Optional[str]
    name: Optional[str]
    xc_language_specification_identifier: Optional[str]
    explicit_file_type: Optional[str]
    include_in_index: Optional[str]


@deserialize.auto_snake()
@deserialize.key("child_ids", "children")
@deserialize.downcast_identifier(PBXObject, "XCVersionGroup")
class XCVersionGroup(PBXPathObject):
    """Represents an XCVersion group

    These groups have versioned files in them
    """

    child_ids: List[str]
    current_version: str
    version_group_type: str
    name: Optional[str]

    @property
    def children(self) -> List[PBXPathObject]:
        """Get all the children for this group.

        :returns: The children for this group
        """
        return [cast(PBXPathObject, self.objects()[child_id]) for child_id in self.child_ids]



"""PBX object types"""

from typing import Any, cast, Dict, List, Optional

import deserialize

from .pbxobject import PBXObject


@deserialize.key("base_configuration_reference_id", "baseConfigurationReference")
@deserialize.key("build_settings", "buildSettings")
@deserialize.downcast_identifier(PBXObject, "XCBuildConfiguration")
class XCBuildConfiguration(PBXObject):
    """Represents an XCBuildConfiguration.

    This is the raw build settings for a build configuration. Name is usually
    "Debug" or "Release".
    """

    base_configuration_reference_id: Optional[str]
    build_settings: Dict[str, Any]
    name: str

    @property
    def base_configuration(self) -> Optional["XCBuildConfiguration"]:
        """Get the base configuration for this build configureation.

        :returns: The base configuration
        """

        if not self.base_configuration_reference_id:
            return None

        return cast(XCBuildConfiguration, self.objects()[self.base_configuration_reference_id])


@deserialize.key("build_configuration_ids", "buildConfigurations")
@deserialize.key("default_configuration_is_visible", "defaultConfigurationIsVisible")
@deserialize.key("default_configuration_name", "defaultConfigurationName")
@deserialize.parser("defaultConfigurationIsVisible", lambda x: {"0": False, "1": True}[x])
@deserialize.downcast_identifier(PBXObject, "XCConfigurationList")
class XCConfigurationList(PBXObject):
    """Represents an XCConfigurationList.

    This is a list of build configurations. Usually associated with a target.
    """

    build_configuration_ids: List[str]
    default_configuration_is_visible: bool
    default_configuration_name: str

    @property
    def build_configurations(self) -> List[XCBuildConfiguration]:
        """Get all the build configurations for this list.

        :returns: The build configurations
        """
        return [
            cast(XCBuildConfiguration, self.objects()[build_configuration_id])
            for build_configuration_id in self.build_configuration_ids
        ]



"""PBX build phases"""

from typing import cast, List, Optional

import deserialize

from .pbxobject import PBXObject
from .files import PBXBuildFile


@deserialize.key("build_action_mask", "buildActionMask")
@deserialize.key("file_ids", "files")
@deserialize.key("run_only_for_deployment_post_processing", "runOnlyForDeploymentPostprocessing")
@deserialize.parser("runOnlyForDeploymentPostprocessing", lambda x: {"0": False, "1": True}[x])
@deserialize.downcast_identifier(PBXObject, "PBXBuildPhase")
class PBXBuildPhase(PBXObject):
    """Represents a PBXBuildPhase."""

    file_ids: List[str]
    build_action_mask: str
    run_only_for_deployment_post_processing: bool

    @property
    def files(self) -> List[PBXBuildFile]:
        """Get the files in the build phase."""
        return [cast(PBXBuildFile, self.objects()[file_id]) for file_id in self.file_ids]


@deserialize.downcast_identifier(PBXObject, "PBXSourcesBuildPhase")
class PBXSourcesBuildPhase(PBXBuildPhase):
    """Represents a PBXSourcesBuildPhase.

    This is a specific build phase for building sources. Several other phases
    exist.
    """


@deserialize.downcast_identifier(PBXObject, "PBXResourcesBuildPhase")
class PBXResourcesBuildPhase(PBXBuildPhase):
    """Represents a PBXResourcesBuildPhase.

    This is a specific build phase for copying resources. Several other phases
    exist.
    """


@deserialize.key("input_paths", "inputPaths")
@deserialize.key("output_paths", "outputPaths")
@deserialize.key("input_file_list_paths", "inputFileListPaths")
@deserialize.key("output_file_list_paths", "outputFileListPaths")
@deserialize.key("shell_path", "shellPath")
@deserialize.key("shell_script", "shellScript")
@deserialize.key("show_env_vars_in_log", "showEnvVarsInLog")
@deserialize.key("always_out_of_date", "alwaysOutOfDate")
@deserialize.key("dependency_file", "dependencyFile")
@deserialize.parser("showEnvVarsInLog", lambda x: {"0": False, "1": True}.get(x, True))
@deserialize.parser("alwaysOutOfDate", lambda x: {"0": False, "1": True}.get(x, True))
@deserialize.downcast_identifier(PBXObject, "PBXShellScriptBuildPhase")
class PBXShellScriptBuildPhase(PBXBuildPhase):
    """Represents a PBXShellScriptBuildPhase.

    This is a specific build phase for running shell scripts resources. Several
    other phases exist.
    """

    input_paths: Optional[List[str]]
    output_paths: Optional[List[str]]
    input_file_list_paths: Optional[List[str]]
    output_file_list_paths: Optional[List[str]]
    name: str
    shell_path: str
    shell_script: str
    show_env_vars_in_log: Optional[bool]
    always_out_of_date: Optional[bool]
    dependency_file: Optional[str]


@deserialize.key("destination_path", "dstPath")
@deserialize.key("destination_subfolder_spec", "dstSubfolderSpec")
@deserialize.parser("dstSubfolderSpec", int)
@deserialize.downcast_identifier(PBXObject, "PBXCopyFilesBuildPhase")
class PBXCopyFilesBuildPhase(PBXBuildPhase):
    """Represents a PBXShellScriptBuildPhase.

    This is a specific build phase for running shell scripts resources. Several
    other phases exist.
    """

    destination_path: Optional[str]
    destination_subfolder_spec: int
    name: Optional[str]


@deserialize.downcast_identifier(PBXObject, "PBXFrameworksBuildPhase")
class PBXFrameworksBuildPhase(PBXBuildPhase):
    """Represents a PBXFrameworksBuildPhase.

    The phase responsible on linking with frameworks. Known as Link Binary With Libraries in the UI.
    """


@deserialize.downcast_identifier(PBXObject, "PBXHeadersBuildPhase")
class PBXHeadersBuildPhase(PBXBuildPhase):
    """Represents a PBXHeadersBuildPhase

    This phase copies headers.
    """



"""PBX object types"""

from typing import Optional

import deserialize

from .pbxobject import PBXObject


@deserialize.auto_snake()
@deserialize.downcast_identifier(PBXObject, "PBXTargetDependency")
class PBXTargetDependency(PBXObject):
    """Represents a PBXTargetDependency.

    Tracks dependencies between targets.
    """

    name: Optional[str]
    target: str
    target_proxy: str


@deserialize.key("remote_global_id_string", "remoteGlobalIDString")
@deserialize.auto_snake()
@deserialize.downcast_identifier(PBXObject, "PBXContainerItemProxy")
class PBXContainerItemProxy(PBXObject):
    """Represents a PBXContainerItemProxy.

    This lets one item be a proxy for another. This is used to link items cross
    project for the most part, but appears to be used in other cases too.
    """

    container_portal: str
    proxy_type: str
    remote_global_id_string: str
    remote_info: str



"""PBXProject object."""

from typing import Any, cast, Dict, List, Optional

import deserialize

from .pathobjects import PBXGroup
from .pbxobject import PBXObject
from .targets import PBXTarget
from .xcobjects import XCConfigurationList


@deserialize.downcast_identifier(PBXObject, "PBXProject")
@deserialize.parser("has_scanned_for_encodings", lambda x: {"0": False, "1": True}[x])
@deserialize.key("target_ids", "targets")
@deserialize.key("main_group_id", "mainGroup")
@deserialize.key("build_configuration_list_id", "buildConfigurationList")
@deserialize.auto_snake()
class PBXProject(PBXObject):
    """Represents a PBXProject.

    This is the root object.
    """

    attributes: Dict[str, Any]
    build_configuration_list_id: str
    compatibility_version: str
    development_region: str
    has_scanned_for_encodings: bool
    known_regions: List[str]
    main_group_id: str
    product_ref_group: Optional[str]
    project_dir_path: str
    project_root: str
    target_ids: List[str]

    @property
    def targets(self) -> List[PBXTarget]:
        """Get the targets in the project."""
        return [cast(PBXTarget, self.objects()[target_id]) for target_id in self.target_ids]

    @property
    def main_group(self) -> PBXGroup:
        """Get the main group in the project."""
        return cast(PBXGroup, self.objects()[self.main_group_id])

    @property
    def build_configuration_list(self) -> XCConfigurationList:
        """Get the build configuration list for the project."""
        return cast(XCConfigurationList, self.objects()[self.build_configuration_list_id])



"""PBX files"""

from typing import Any, cast, Dict, Optional

import deserialize

from .pbxobject import PBXObject
from .pathobjects import PBXFileReference


@deserialize.key("file_ref_id", "fileRef")
@deserialize.key("platform_filter", "platformFilter")
@deserialize.downcast_identifier(PBXObject, "PBXBuildFile")
class PBXBuildFile(PBXObject):
    """Represents a PBXBuildFile.

    These are files which are marked as for building in a phase, and are
    effectively pointers to a PBXFileReference which is an actual file.
    """

    file_ref_id: str
    platform_filter: Optional[str]
    settings: Optional[Dict[str, Any]]

    @property
    def file_ref(self) -> PBXFileReference:
        """Get the file reference.

        :returns: The file reference
        """
        return cast(PBXFileReference, self.objects()[self.file_ref_id])



"""PBX object types"""

from typing import Any, Callable, cast, Dict

import deserialize


@deserialize.allow_unhandled("isa")
@deserialize.downcast_field("isa")
@deserialize.allow_downcast_fallback()
@deserialize.ignore("objects_ref")
@deserialize.ignore("project_ref")
class PBXObject:
    """Base class for an object in a PBX Project file.

    :param object_key: The key for the object.
    """

    object_key: str
    objects_ref: Callable[[], Dict[str, "PBXObject"]]
    project_ref: Callable[[], Any]

    def __getstate__(self):
        """Return state values to be pickled."""
        new_dict = {}
        for key, value in self.__dict__.items():
            if key in ["objects_ref", "project_ref"]:
                continue
            new_dict[key] = value
        return new_dict

    def objects(self) -> Dict[str, "PBXObject"]:
        """Resolve objects reference.

        :returns: Resolved objects dictionary
        """
        reference = getattr(self, "objects_ref")
        return cast(Dict[str, "PBXObject"], reference())

    def project(self) -> Any:
        """Resolve objects reference.

        :returns: Resolved objects dictionary
        """
        reference = getattr(self, "project_ref")
        return reference()

    def __eq__(self, other: object) -> bool:
        """Determine if the supplied object is equal to self.

        :param other: The object to compare to self

        :returns: True if they are equal, False otherwise.
        """

        if not isinstance(other, type(self)):
            return False

        return self.object_key == other.object_key

    def __hash__(self) -> int:
        """Calculate the hash of the object

        :returns: The hash value of the object
        """

        return hash(self.object_key)



"""Xcode project file management."""

try:
    import _pickle as pickle
except ImportError:
    import pickle  # type: ignore

from collections import defaultdict
import functools
import hashlib
import json
import os
import pathlib
from typing import (
    Any,
    cast,
    Dict,
    List,
    MutableMapping,
    Optional,
    Type,
    TypeVar,
)
import subprocess
import weakref

import appdirs
import deserialize

from .pbxobject import PBXObject
from .buildphases import (
    PBXBuildPhase,
    PBXCopyFilesBuildPhase,
    PBXFrameworksBuildPhase,
    PBXHeadersBuildPhase,
    PBXResourcesBuildPhase,
    PBXShellScriptBuildPhase,
    PBXSourcesBuildPhase,
)
from .buildrules import PBXBuildRule
from .files import PBXBuildFile
from .pathobjects import (
    PBXPathObject,
    PBXGroup,
    PBXVariantGroup,
    PBXFileReference,
    XCVersionGroup,
)
from .pbxproject import PBXProject
from .other import PBXTargetDependency, PBXContainerItemProxy
from .schemes import Scheme
from .targets import PBXAggregateTarget, PBXNativeTarget, PBXProductType
from .xcobjects import XCBuildConfiguration, XCConfigurationList


PBXObjectType = TypeVar("PBXObjectType", bound=PBXObject)


def _load_pbxproj_as_json(path: str) -> Dict[str, Any]:
    """Load a pbxproj as JSON.

    :param path: The path to the pbxproj

    :returns: A deserialized representation of the pbxproj
    """

    content = subprocess.run(
        [
            "plutil",
            "-convert",
            "json",
            os.path.join(path, "project.pbxproj"),
            "-o",
            "-",
        ],
        stdout=subprocess.PIPE,
        check=True,
    ).stdout
    return cast(Dict[str, Any], json.loads(content))


class Objects(dict, MutableMapping[str, PBXObject]):
    """Holds the objects in the pbxproj."""


class XcodeProject:
    """Represents an Xcodeproject.

    :param path: The path to the pbxproj file
    """

    path: str
    source_root: str
    objects: Objects
    project: PBXProject
    _cached_items: Dict[str, Dict[str, PBXObject]]
    _schemes: Optional[List[Scheme]]
    _is_populated: bool

    def __init__(self, path: str) -> None:
        self.path = path
        self.source_root = os.path.dirname(path)
        tree = _load_pbxproj_as_json(path)

        for key, value in tree["objects"].items():
            value["object_key"] = key

        self.objects = Objects(
            **deserialize.deserialize(
                Dict[str, PBXObject],
                tree["objects"],
                throw_on_unhandled=True,
                raw_storage_mode=deserialize.RawStorageMode.all,
            )
        )

        self.project = self.objects[tree["rootObject"]]
        self._cached_items = {}
        self._schemes = None
        self._is_populated = False

        self._set_weak_refs()

    @staticmethod
    def from_cache(project_path: str) -> "XcodeProject":
        """Attempt to load the project from a cached folder if possible.

        :param project_path: The path to the actual project (in case it's a cache miss)

        :returns: The loaded XcodeProj
        """
        cache_folder = appdirs.user_cache_dir("xcodeproj")

        try:
            project_hash = hashlib.md5(
                pathlib.Path(os.path.join(project_path, "project.pbxproj")).read_bytes()
            ).hexdigest()

            with open(os.path.join(cache_folder, f"{project_hash}.dat"), "rb") as cached_file:
                return pickle.load(cached_file)
        except Exception:
            return XcodeProject(project_path)

    def write_cache(self) -> None:
        """Write out this file to a cache

        :param cache_folder: The folder to store the cached project in.
        """
        self.populate_paths()

        cache_folder = appdirs.user_cache_dir("xcodeproj")
        os.makedirs(cache_folder, exist_ok=True)

        project_hash = hashlib.md5(
            pathlib.Path(os.path.join(self.path, "project.pbxproj")).read_bytes()
        ).hexdigest()

        with open(os.path.join(cache_folder, f"{project_hash}.dat"), "wb") as cached_file:
            pickle.dump(self, cached_file)

    def __setstate__(self, state):
        """Restore state from the unpickled state values."""
        self.__dict__ = state
        self._set_weak_refs()

    def _set_weak_refs(self) -> None:
        """Setup the weak references."""
        for obj in self.objects.values():
            obj.objects_ref = weakref.ref(self.objects)
            obj.project_ref = weakref.ref(self)

    def _populate_cache(self, object_type: Type[PBXObject]) -> None:
        """Populate the cache of items specified.

        This just avoids a full dictionary scan

        :param object_type: The type of objects to populate
        """

        if object_type.__name__ in self._cached_items:
            return

        cached_items: Dict[str, PBXObject] = {}

        for object_key, project_object in self.objects.items():

            if not isinstance(project_object, object_type):
                continue

            cached_items[object_key] = project_object

        self._cached_items[object_type.__name__] = cached_items

    def populate_paths(self) -> None:
        """Pre-emptively populate group paths.

        This method is from the top down so is much quicker.
        """

        if self._is_populated:
            return

        non_set = []

        def populate(parent_group: PBXPathObject, path: Optional[str]) -> None:
            nonlocal non_set

            if path is not None:
                setattr(parent_group, "_relative_path", path)

            if not isinstance(parent_group, PBXGroup):
                return

            for subgroup in parent_group.children:
                if subgroup.source_tree == "SOURCE_ROOT":
                    populate(subgroup, subgroup.path)
                elif subgroup.source_tree == "<group>":
                    if subgroup.path is None:
                        if path is None:
                            populate(subgroup, path)
                        else:
                            non_set.append(subgroup)
                    else:
                        if path is not None:
                            populate(subgroup, os.path.join(path, subgroup.path))
                        else:
                            populate(subgroup, subgroup.path)
                else:
                    non_set.append(subgroup)

        root_group = self.objects[self.project.main_group_id]
        populate(root_group, None)

        for item in non_set:
            _ = item.relative_path()

        self._is_populated = True

    def fetch_type(self, object_type: Type[PBXObjectType]) -> Dict[str, PBXObjectType]:
        """Load the items specified from the cache, populating the cache if required.

        :param object_type: The type of objects to get

        :returns: The type from the cache
        """
        self._populate_cache(object_type)
        return cast(Dict[str, PBXObjectType], self._cached_items[object_type.__name__])

    def targets(self) -> List[PBXNativeTarget]:
        """Get the targets.

        :returns: A list of targets
        """
        return list(self.fetch_type(PBXNativeTarget).values())

    def target_by_name(self, name: str) -> Optional[PBXNativeTarget]:
        """Get a target by name.

        :param str name: The name of the target to find

        :returns: The target if found, else None
        """
        for target in self.fetch_type(PBXNativeTarget).values():
            if target.name == name:
                return target
        return None

    def target_containing_phase(self, source_build_phase_ref: str) -> Optional[PBXNativeTarget]:
        """Find the target containing the Source Build Phase
        :param source_build_phase_ref: The reference of the parent

        :returns: The target containing the source build phase
        """
        for native_target in self.fetch_type(PBXNativeTarget).values():
            for build_phase in native_target.build_phases:
                if build_phase.object_key == source_build_phase_ref:
                    return native_target
        return None

    def build_configuration_list_for_target(
        self, native_target_name: str
    ) -> Optional[XCConfigurationList]:
        """Searches for build configuration via a target's name

        :param native_target_name: The name of the native target to find

        :returns: The build configuration
        """

        for native_target in self.fetch_type(PBXNativeTarget).values():
            if native_target_name != native_target.name:
                continue

            return native_target.build_configuration_list

        return None

    @property
    def schemes(self) -> List[Scheme]:
        """Load the schemes for the project.

        :returns: A list of schemes
        """
        if self._schemes is not None:
            return self._schemes

        scheme_paths = []

        for path, _, files in os.walk(self.path):
            for file_path in files:
                if not file_path.endswith(".xcscheme"):
                    continue
                scheme_paths.append(os.path.join(path, file_path))

        all_schemes: List[Scheme] = []

        for scheme_path in scheme_paths:
            all_schemes.append(Scheme.from_file(scheme_path))

        self._schemes = all_schemes

        return all_schemes



"""PBX object types"""

import enum
from typing import cast, List, Optional

import deserialize

from .pbxobject import PBXObject
from .buildphases import PBXBuildPhase
from .buildrules import PBXBuildRule
from .pathobjects import PBXFileReference
from .xcobjects import XCConfigurationList


class PBXProductType(enum.Enum):
    """The different product types available."""

    # fmt: off
    application          = "com.apple.product-type.application"
    bundle               = "com.apple.product-type.bundle"
    uitesting_bundle     = "com.apple.product-type.bundle.ui-testing"
    unit_test            = "com.apple.product-type.bundle.unit-test"
    static_library       = "com.apple.product-type.library.static"
    library_bundle       = "com.apple.product-type.library.bundle"
    framework            = "com.apple.product-type.framework"
    app_extension        = "com.apple.product-type.app-extension"
    watch_app2           = "com.apple.product-type.application.watchapp2"
    watch_app2_container = "com.apple.product-type.application.watchapp2-container"
    watchkit2_extension  = "com.apple.product-type.watchkit2-extension"
    # fmt: on


@deserialize.key("build_phases_ids", "buildPhases")
@deserialize.key("build_configuration_list_id", "buildConfigurationList")
@deserialize.key("dependency_ids", "dependencies")
@deserialize.auto_snake()
class PBXTarget(PBXObject):
    """Represents a PBXTarget."""

    build_configuration_list_id: str
    build_phases_ids: List[str]
    dependency_ids: List[str]
    name: str
    product_name: Optional[str]

    @property
    def build_phases(self) -> List[PBXBuildPhase]:
        """Get the build phases in the target."""
        return [cast(PBXBuildPhase, self.objects()[phase_id]) for phase_id in self.build_phases_ids]

    @property
    def build_configuration_list(self) -> XCConfigurationList:
        """Get the build configuration list for the target."""
        return cast(XCConfigurationList, self.objects()[self.build_configuration_list_id])

    @property
    def dependencies(self) -> List["PBXTarget"]:
        """Get the dependencies of the target."""
        return [
            cast(PBXTarget, self.objects()[dependency_id]) for dependency_id in self.dependency_ids
        ]


@deserialize.downcast_identifier(PBXObject, "PBXAggregateTarget")
class PBXAggregateTarget(PBXTarget):
    """Represents a PBXAggregateTarget."""


@deserialize.auto_snake()
@deserialize.key("product_reference_id", "productReference")
@deserialize.key("build_rule_ids", "buildRules")
@deserialize.downcast_identifier(PBXObject, "PBXNativeTarget")
class PBXNativeTarget(PBXTarget):
    """Represents a PBXNativeTarget.

    This is the corresponding target type to an aggregate target.
    """

    build_rule_ids: Optional[List[str]]
    product_reference_id: Optional[str]
    product_type: PBXProductType

    @property
    def product_reference(self) -> Optional[PBXFileReference]:
        """Get the product reference of the target."""
        if self.product_reference_id is None:
            return None
        return cast(PBXFileReference, self.objects()[self.product_reference_id])

    @property
    def build_rules(self) -> List[PBXBuildRule]:
        """Get the product reference of the target."""
        if self.build_rule_ids is None:
            return []

        return [
            cast(PBXBuildRule, self.objects()[build_rule_id])
            for build_rule_id in self.build_rule_ids
        ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xcodeproj']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.0,<2.0', 'deserialize>=1.8.0,<2.0.0']

setup_kwargs = {
    'name': 'xcodeproj',
    'version': '0.10.0',
    'description': "A utility for interacting with Xcode's xcodeproj bundle format.",
    'long_description': '# xcodeproj\n\n`xcodeproj` is a utility for interacting with Xcode\'s xcodeproj bundle format.\n\nIt expects some level of understanding of the internals of the pbxproj format and schemes. Note that this tool only reads projects. It does not write out any changes. If you are looking for more advanced functionality like this, I recommend looking at the Ruby gem of the same name (which is unaffiliated in anyway). \n\nTo learn more about the format, you can look at any of these locations:\n\n* <http://www.monobjc.net/xcode-project-file-format.html>\n* <https://www.rubydoc.info/gems/xcodeproj/Xcodeproj/Project>\n\n## Getting Started\n\nLoading a project is very simple:\n\n```python\nproject = xcodeproj.XcodeProject("/path/to/project.xcodeproj")\n```\n\nFrom here you can explore the project in different ways:\n\n```python\n\n# Get all targets\nfor target in project.targets:\n    print(target.name)\n\n# Print from the root level, 2 levels deep (.project is a property on the root \n# project as other properties such as .schemes are also available)\nfor item1 in project.project.main_group.children:\n    print(item1)\n    if not isinstance(item1, xcodeproj.PBXGroup):\n        continue\n\n    for item2 in item1.children:\n        print("\\t", item2)\n\n# Check that all files referenced in the project exist on disk\nfor item in project.fetch_type(xcodeproj.PBXFileReference).values():\n    assert os.path.exists(item.absolute_path())\n\n# You can access the raw objects map directly:\nobj = project.objects["key here"]\n\n# For any object you have, you can access its key/identifier via the \n# `.object_key` property\nkey = obj.object_key\n```\n\nNote: This library is "lazy". Many things aren\'t calculated until they are used. This time will be inconsequential on smaller projects, but on larger ones, it can save quite a bit of time due to not parsing the entire project on load. These properties are usually stored though so that subsequent accesses are instant.\n\n## Note on Scheme Support\nThere\'s no DTD for xcscheme files, so the implementation has been guessed. There will definitely be holes that still need to be patched in it though. Please open an issue if you find any, along with a sample xcscheme file.\n\n## Contributing\n\nThis project welcomes contributions and suggestions.  Most contributions require you to agree to a\nContributor License Agreement (CLA) declaring that you have the right to, and actually do, grant us\nthe rights to use your contribution. For details, visit https://cla.opensource.microsoft.com.\n\nWhen you submit a pull request, a CLA bot will automatically determine whether you need to provide\na CLA and decorate the PR appropriately (e.g., status check, comment). Simply follow the instructions\nprovided by the bot. You will only need to do this once across all repos using our CLA.\n\nThis project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/).\nFor more information see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or\ncontact [opencode@microsoft.com](mailto:opencode@microsoft.com) with any additional questions or comments.\n\n## Trademarks\n\nThis project may contain trademarks or logos for projects, products, or services. Authorized use of Microsoft \ntrademarks or logos is subject to and must follow \n[Microsoft\'s Trademark & Brand Guidelines](https://www.microsoft.com/en-us/legal/intellectualproperty/trademarks/usage/general).\nUse of Microsoft trademarks or logos in modified versions of this project must not cause confusion or imply Microsoft sponsorship.\nAny use of third-party trademarks or logos are subject to those third-party\'s policies.\n',
    'author': 'Dale Myers',
    'author_email': 'dalemy@microsoft.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Microsoft/xcodeproj',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
